@@ -1205,16 +1205,14 @@ on_volume_changed_main_context (gpointer data)
 {
   ClutterGstPlayer *player = CLUTTER_GST_PLAYER (data);
   ClutterGstPlayerPrivate *priv = PLAYER_GET_PRIVATE (player);
+  gdouble volume;
 
-  if (priv)
-    {
-      gdouble volume =
-        gst_stream_volume_get_volume (GST_STREAM_VOLUME (priv->pipeline),
-                                      GST_STREAM_VOLUME_FORMAT_CUBIC);
-      priv->volume = volume;
+  volume =
+    gst_stream_volume_get_volume (GST_STREAM_VOLUME (priv->pipeline),
+                                  GST_STREAM_VOLUME_FORMAT_CUBIC);
+  priv->volume = volume;
 
-      g_object_notify (G_OBJECT (player), "audio-volume");
-    }
+  g_object_notify (G_OBJECT (player), "audio-volume");
 
   g_object_unref (player);
 
@@ -1263,15 +1261,12 @@ on_audio_changed_main_context (gpointer data)
   ClutterGstPlayer *player = CLUTTER_GST_PLAYER (data);
   ClutterGstPlayerPrivate *priv = PLAYER_GET_PRIVATE (player);
 
-  if (priv)
-    {
-      free_tags_list (&priv->audio_streams);
-      priv->audio_streams = get_tags (priv->pipeline, "n-audio", "get-audio-tags");
+  free_tags_list (&priv->audio_streams);
+  priv->audio_streams = get_tags (priv->pipeline, "n-audio", "get-audio-tags");
 
-      CLUTTER_GST_NOTE (AUDIO_STREAM, "audio-streams changed");
+  CLUTTER_GST_NOTE (AUDIO_STREAM, "audio-streams changed");
 
-      g_object_notify (G_OBJECT (player), "audio-streams");
-    }
+  g_object_notify (G_OBJECT (player), "audio-streams");
 
   g_object_unref (player);
 
@@ -1328,15 +1323,12 @@ on_text_changed_main_context (gpointer data)
   ClutterGstPlayer *player = CLUTTER_GST_PLAYER (data);
   ClutterGstPlayerPrivate *priv = PLAYER_GET_PRIVATE (player);
 
-  if (priv)
-    {
-      free_tags_list (&priv->subtitle_tracks);
-      priv->subtitle_tracks = get_tags (priv->pipeline, "n-text", "get-text-tags");
+  free_tags_list (&priv->subtitle_tracks);
+  priv->subtitle_tracks = get_tags (priv->pipeline, "n-text", "get-text-tags");
 
-      CLUTTER_GST_NOTE (AUDIO_STREAM, "subtitle-tracks changed");
+  CLUTTER_GST_NOTE (AUDIO_STREAM, "subtitle-tracks changed");
 
-      g_object_notify (G_OBJECT (player), "subtitle-tracks");
-    }
+  g_object_notify (G_OBJECT (player), "subtitle-tracks");
 
   g_object_unref (player);
 
@@ -2083,9 +2075,8 @@ clutter_gst_player_init (ClutterGstPlayer *player)
                     player);
 
 #if defined(CLUTTER_WINDOWING_X11) && defined (HAVE_HW_DECODER_SUPPORT)
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11))
-    gst_bus_set_sync_handler (priv->bus, on_sync_message,
-                              clutter_x11_get_default_display (), NULL);
+  gst_bus_set_sync_handler (priv->bus, on_sync_message,
+      clutter_x11_get_default_display (), NULL);
 #endif
 
   gst_object_unref (GST_OBJECT (priv->bus));
