@@ -59,6 +59,11 @@
 #include <gst/video/navigation.h>
 #include <gst/riff/riff-ids.h>
 
+#ifdef CLUTTER_WINDOWING_X11
+#include <cogl/cogl-texture-pixmap-x11.h>
+#include <clutter/x11/clutter-x11.h>
+#endif
+
 #ifdef HAVE_HW_DECODER_SUPPORT
 #define GST_USE_UNSTABLE_API 1
 #include <gst/video/gstsurfacemeta.h>
@@ -213,6 +218,10 @@ struct _ClutterGstVideoSinkPrivate
 
 #ifdef HAVE_HW_DECODER_SUPPORT
   GstSurfaceConverter *converter;
+
+#ifdef CLUTTER_WINDOWING_X11
+  Pixmap pixmap;
+#endif
 #endif
 };
 
@@ -1021,7 +1030,6 @@ clutter_gst_ayuv_upload (ClutterGstVideoSink * sink, GstBuffer * buffer)
   ClutterGstVideoSinkPrivate *priv = sink->priv;
   CoglHandle tex;
   GstVideoFrame frame;
-  GstMapInfo info;
 
   if (!gst_video_frame_map (&frame, &priv->info, buffer, GST_MAP_READ))
     goto map_fail;
@@ -1062,23 +1070,105 @@ static ClutterGstRenderer ayuv_glsl_renderer = {
  */
 
 #ifdef HAVE_HW_DECODER_SUPPORT
-static void
-clutter_gst_hw_init (ClutterGstVideoSink * sink)
+static gboolean
+clutter_gst_hw_set_texture (ClutterGstVideoSink * sink, CoglTexture * tex)
 {
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-  CoglHandle tex;
+  ClutterGstVideoSinkPrivate * const priv = sink->priv;
   CoglHandle material;
 
-  /* Default texture is 1x1, let's replace it with one big enough. */
-  tex = cogl_texture_new_with_size (priv->info.width, priv->info.height,
-      CLUTTER_GST_TEXTURE_FLAGS, COGL_PIXEL_FORMAT_BGRA_8888);
-
   material = cogl_material_new ();
+  if (!material)
+    return FALSE;
   cogl_material_set_layer (material, 0, tex);
   clutter_texture_set_cogl_material (priv->texture, material);
 
   cogl_object_unref (tex);
   cogl_object_unref (material);
+  return TRUE;
+}
+
+static gboolean
+clutter_gst_hw_init_texture (ClutterGstVideoSink * sink,
+    GstSurfaceMeta * surface, GstBuffer * buffer)
+{
+  ClutterGstVideoSinkPrivate * const priv = sink->priv;
+  CoglHandle tex;
+  unsigned int gl_texture;
+  unsigned int gl_target;
+  GValue value = { 0 };
+
+  /* Default texture is 1x1, let's replace it with one big enough. */
+  tex = cogl_texture_new_with_size (priv->info.width, priv->info.height,
+      CLUTTER_GST_TEXTURE_FLAGS, COGL_PIXEL_FORMAT_BGRA_8888);
+  if (!tex)
+    return FALSE;
+
+  if (!clutter_gst_hw_set_texture (sink, tex)) {
+    cogl_object_unref (tex);
+    return FALSE;
+  }
+
+  cogl_texture_get_gl_texture (tex, &gl_texture, &gl_target);
+
+  g_value_init (&value, G_TYPE_UINT);
+  g_value_set_uint (&value, gl_texture);
+
+  priv->converter =
+    gst_surface_meta_create_converter (surface, "opengl", &value);
+  return priv->converter != NULL;
+}
+
+static gboolean
+clutter_gst_hw_init_pixmap (ClutterGstVideoSink * sink,
+    GstSurfaceMeta * surface, GstBuffer * buffer)
+{
+#ifdef CLUTTER_WINDOWING_X11
+  ClutterGstVideoSinkPrivate * const priv = sink->priv;
+  Display * const dpy = clutter_x11_get_default_display ();
+  int screen = clutter_x11_get_default_screen ();
+  ClutterBackend *backend;
+  CoglContext *context;
+  CoglHandle tex;
+  GValue value = { 0 };
+
+  priv->pixmap = XCreatePixmap(dpy, clutter_x11_get_root_window (),
+      priv->info.width, priv->info.height, DefaultDepth (dpy, screen));
+  if (!priv->pixmap)
+    return FALSE;
+
+  backend = clutter_get_default_backend ();
+  context = clutter_backend_get_cogl_context (backend);
+  tex = cogl_texture_pixmap_x11_new (context, priv->pixmap, FALSE, NULL);
+  if (!tex)
+    goto error;
+  if (!cogl_texture_pixmap_x11_is_using_tfp_extension (tex))
+    goto error;
+  if (!clutter_gst_hw_set_texture (sink, tex))
+    goto error;
+
+  g_value_init (&value, G_TYPE_UINT);
+  g_value_set_uint (&value, priv->pixmap);
+
+  priv->converter =
+    gst_surface_meta_create_converter (surface, "x11-pixmap", &value);
+  if (!priv->converter)
+    goto error;
+  return TRUE;
+
+  /* ERRORS */
+ error:
+  if (tex)
+    cogl_object_unref (tex);
+  XFreePixmap (dpy, priv->pixmap);
+  priv->pixmap = None;
+  return FALSE;
+#endif
+  return FALSE;
+}
+
+static void
+clutter_gst_hw_init (ClutterGstVideoSink * sink)
+{
 }
 
 static void
@@ -1086,6 +1176,13 @@ clutter_gst_hw_deinit (ClutterGstVideoSink * sink)
 {
   ClutterGstVideoSinkPrivate *priv = sink->priv;
 
+#ifdef CLUTTER_WINDOWING_X11
+  if (priv->pixmap != None) {
+    XFreePixmap (clutter_x11_get_default_display (), priv->pixmap);
+    priv->pixmap = None;
+  }
+#endif
+
   if (priv->converter != NULL)
     g_object_unref (priv->converter);
   priv->converter = NULL;
@@ -1100,19 +1197,12 @@ clutter_gst_hw_upload (ClutterGstVideoSink * sink, GstBuffer * buffer)
   g_return_val_if_fail (surface != NULL, FALSE);
 
   if (G_UNLIKELY (priv->converter == NULL)) {
-    CoglHandle tex;
-    unsigned int gl_texture;
-    unsigned int gl_target;
-    GValue value = { 0 };
-
-    tex = clutter_texture_get_cogl_texture (priv->texture);
-    cogl_texture_get_gl_texture (tex, &gl_texture, &gl_target);
-
-    g_value_init (&value, G_TYPE_UINT);
-    g_value_set_uint (&value, gl_texture);
-
-    priv->converter =
-        gst_surface_meta_create_converter (surface, "opengl", &value);
+    do {
+      if (clutter_gst_hw_init_pixmap (sink, surface, buffer))
+        break;
+      if (clutter_gst_hw_init_texture (sink, surface, buffer))
+        break;
+    } while (0);
     g_return_val_if_fail (priv->converter, FALSE);
   }
 
@@ -1217,7 +1307,7 @@ navigation_event (ClutterActor * actor,
   if (event->type == CLUTTER_MOTION) {
     ClutterMotionEvent *mevent = (ClutterMotionEvent *) event;
 
-    GST_DEBUG ("Received mouse move event to %d,%d", mevent->x, mevent->y);
+    GST_DEBUG ("Received mouse move event to %.2f,%.2f", mevent->x, mevent->y);
     gst_navigation_send_mouse_event (GST_NAVIGATION (sink),
         "mouse-move", 0, mevent->x, mevent->y);
   } else if (event->type == CLUTTER_BUTTON_PRESS ||
@@ -1225,7 +1315,7 @@ navigation_event (ClutterActor * actor,
     ClutterButtonEvent *bevent = (ClutterButtonEvent *) event;
     const char *type;
 
-    GST_DEBUG ("Received button %s event at %d,%d",
+    GST_DEBUG ("Received button %s event at %.2f,%.2f",
         (event->type == CLUTTER_BUTTON_PRESS) ? "press" : "release",
         bevent->x, bevent->y);
     type =
